<?php

if ( ! class_exists('CtMPAC_Application') ) {
	class CtMPAC_Application {
		public function __construct() {
			add_action( 'woocommerce_check_cart_items', array($this, 'ct_mpac_set_minimum_requirements'));
			add_action( 'woocommerce_init', array($this, 'register_wc_shortcodes'));
			add_action( 'ct_mpac_filter_min_cart_total', array($this, 'filter_minimum_order_amount_based_on_the_roles'), 11 , 2);
			add_action( 'woocommerce_proceed_to_checkout', array($this, 'disable_checkout_link'), 1);
			add_filter( 'ct_mpac_filter_current_cart_total' , array( $this , 'ct_mpac_exclude_shipping_in_cart_total' ), 12, 2);
		}


		public function disable_checkout_link() {
			$checkOutSetting = get_option('ct_mpac_cart_disable_checkout' , false);
			if ( $checkOutSetting ) {
				$minimumCartTotal = $this->get_minimum_purchase_value();
				$currentCartTotal = $this->get_current_cart_total();
				if ( 0!==$currentCartTotal && $currentCartTotal < $minimumCartTotal  ) {
					remove_action( 'woocommerce_proceed_to_checkout', 'woocommerce_button_proceed_to_checkout', 20 );
					echo '<a href="#" class="checkout-button button alt wc-forward">';
					esc_html_e('Proceed to checkout', 'ct-minimum-purchase-amount-for-woo-cart');
					echo '</a>';       
				}
			}
		}

		public function get_minimum_purchase_value() {
			$minimumCartTotal = get_option( 'ct_mpac_minimum_purchase_value_for_all', 0);
			/**
			 * Filter the amount set as the minimum cart total to customize the behaviour with this hook.
			 * 
			 * @since 2.2
			 */
			$minimumCartTotal = apply_filters('ct_mpac_filter_min_cart_total', $minimumCartTotal, WC()->cart);
			return $minimumCartTotal;
		}


		public function get_current_cart_total() {
			$cartTotal = WC()->cart->total;
		
			if (get_option('ct_mpac_include_discount_coupons_in_total', false)) {
				//If Consider discount in total setting is enabled.
				$cartTotal = WC()->cart->subtotal;
			}
			/**
			 * Filter the cart total , value which is considered while checking if it matches the minimum cart amount
			 * set from the setting.
			 * 
			 * @since 2.2
			 */
			return apply_filters('ct_mpac_filter_current_cart_total', $cartTotal, WC()->cart);
		}

		public function ct_mpac_set_minimum_requirements() {			
			global $woocommerce;
			$minimumCartTotal = $this->get_minimum_purchase_value();
			$currentCartTotal = $this->get_current_cart_total();
			if ( 0!==$currentCartTotal && $currentCartTotal < $minimumCartTotal  ) {
				$customCartMessage = self::getErrorNotice($minimumCartTotal, $currentCartTotal);
				if ( is_cart()) {
					wc_print_notice($customCartMessage, 'error');
				} else {
					wc_add_notice($customCartMessage, 'error');
				}
			}			
		}

		public static function getErrorNotice( $minimumCartTotal, $currentCartTotal) {
			if (!empty($minimumCartTotal) && 0<$minimumCartTotal) {
				$remainingAmountToMakePurchase = '<span class="ct-mpac-remaining-amount">' . wc_price($minimumCartTotal - $currentCartTotal) . '</span>'; 
				$minimumCartTotal              = '<span class="ct-mpac-minimum-amount">' . wc_price($minimumCartTotal) . '</span>';
				$currentCartTotal			   = '<span class="ct-mpac-current-amount">' . wc_price($currentCartTotal) . '</span>';
				$customCartMessage             = get_option('ct_mpac_cart_limit_message', null);
				//load default message
				if (empty($customCartMessage) || null===$customCartMessage) {
					$customCartMessage = __('The minimum amount to make a purchase is [minimum-amount] & the current cart total is [current-total] please add the products worth [amount-remaining] or more to successfully make a purchase', 'ct-minimum-purchase-amount-for-woo-cart');
				}

				$customCartMessage = str_replace(__('[minimum-amount]', 'ct-minimum-purchase-amount-for-woo-cart'), $minimumCartTotal, $customCartMessage);
				$customCartMessage = str_replace('[minimum-amount]', $minimumCartTotal, $customCartMessage);
				$customCartMessage = str_replace(__('[amount-remaining]', 'ct-minimum-purchase-amount-for-woo-cart'), $remainingAmountToMakePurchase, $customCartMessage);
				$customCartMessage = str_replace('[amount-remaining]', $remainingAmountToMakePurchase, $customCartMessage);
				$customCartMessage = str_replace(__('[current-total]', 'ct-minimum-purchase-amount-for-woo-cart'), $currentCartTotal, $customCartMessage);
				$customCartMessage = str_replace('[current-total]', $currentCartTotal, $customCartMessage);
				return $customCartMessage;
			}
		}


		public function register_wc_shortcodes() {
			add_shortcode( 'ct_mpac_minimum_order_amount_message', array($this, 'ct_mpac_minimum_order_amount_shortcode'));
			$this->register_styles_and_scripts();
		}

		public function ct_mpac_minimum_order_amount_shortcode() {
			ob_start();

			if (!is_admin()) {
				if (WC() && null!==WC()->cart) {
					$minimumCartTotal = get_option( 'ct_mpac_minimum_purchase_value_for_all', 0);
					/**
					* Filter the amount set as the minimum cart total to customize the behaviour with this hook.
					* 
					* @since 2.2
					*/
					$minimumCartTotal = apply_filters('ct_mpac_filter_min_cart_total', $minimumCartTotal, WC()->cart);
					$currentCartTotal = $this->get_current_cart_total();
					if ( 0!==$currentCartTotal && $minimumCartTotal>$currentCartTotal) {
						$noticeText = self::getErrorNotice($minimumCartTotal, $currentCartTotal);
						include_once CT_MPAC_DIR_PATH . '/templates/ct-mpac-minimum-amount-notice.php';
						wp_enqueue_style('ct-mpac-min-amount-notice');
					}
				}
			}
			return ob_get_clean();
		}

		public function register_styles_and_scripts() {
			wp_register_style( 'ct-mpac-min-amount-notice', CT_MPAC_DIR_URL . '/assets/css/shortcode-notice.css', false, CT_MPAC_VERSION);
		}

		public function filter_minimum_order_amount_based_on_the_roles( $minimumOrderAmount, $cart) {
			$roleBasedLimits = get_option( 'ct-mpac-role-specific-cart-limits', array());
			$userRoles       = self::getCurrentUsersRoles();
			$limits          = array();
			if (!empty($roleBasedLimits)) {
				foreach ($roleBasedLimits as $role => $settings) {
					if (in_array($role, $userRoles) && ( isset($settings['status']) && $settings['status'] )) {
						$limits[] = $settings['minimum_order_amount'];
					}
				}
			}
			if (!empty($limits)) {
				/**
			 * When the user has multiple roles & minimum order amount is set for each of the roles,
			 * this filters allow you to specify wether to consider the smallest amount or the largest amount
			 * as the minimum order amount for that user.
			 * 
			 * @since 2.2
			 */
				$minOrMaxRoleBasedMinimumOrderAmount = apply_filters( 'ct-mpac-min-max-minimum-order-amount-from-multiple-roles', 'min');
				$minimumOrderAmount                  = 'min'===$minOrMaxRoleBasedMinimumOrderAmount?min($limits):max($limits);
			}
			return $minimumOrderAmount;
		}

		public static function getCurrentUsersRoles() {
			$userRoles = array();
			if (is_user_logged_in()) {
				$user      = wp_get_current_user();
				$userRoles = $user->roles;
			}
			return $userRoles;
		}

		public function ct_mpac_exclude_shipping_in_cart_total( $cartTotal, $cart) {
			if ( !empty( $cart ) && ! empty( $cart->shipping_total ) && get_option( 'ct_mpac_exclude_shipping_from_cart_total' , false ) ) {
				$cartTotal = $cart->subtotal;
			}
			return $cartTotal;
		}
	}
}
